@@ -91,8 +91,8 @@ shakeArgs opts rules = shakeArgsWith opts [] f
 -- 'shakeArgsWith' opts flags (\\flagValues argValues -> result)
 -- @
 --
--- * @opts@ is the initial 'ShakeOptions' value, which may have some fields overriden by command line flags.
---   This argument is usually 'shakeOptions', perhaps with a few fields overriden.
+-- * @opts@ is the initial 'ShakeOptions' value, which may have some fields overridden by command line flags.
+--   This argument is usually 'shakeOptions', perhaps with a few fields overridden.
 --
 -- * @flags@ is a list of flag descriptions, which either produce a 'String' containing an error
 --   message (typically for flags with invalid arguments, .e.g. @'Left' \"could not parse as int\"@), or a value
@@ -254,7 +254,7 @@ shakeArgsOptionsWith baseOpts userOptions rules = do
         if not ran || shakeVerbosity shakeOpts < Info || NoTime `elem` flagsExtra then
             either throwIO pure res
          else
-            let esc = if shakeColor shakeOpts then escape else flip const
+            let esc = if shakeColor shakeOpts then escape else \_ x -> x
             in case res of
                 Left err ->
                     if Exception `elem` flagsExtra then
@@ -310,6 +310,8 @@ outputColor output v msg = output v $ color msg
 shakeOptsEx :: [(Bool, OptDescr (Either String ([Extra], ShakeOptions -> ShakeOptions)))]
 shakeOptsEx =
     [opts $ Option "a" ["abbrev"] (reqArgPair "abbrev" "FULL=SHORT" $ \a s -> s{shakeAbbreviations=shakeAbbreviations s ++ [a]}) "Use abbreviation in status messages."
+    ,opts $ Option ""  ["allow-redefine-rules"] (noArg $ \s -> s{shakeAllowRedefineRules = True}) "Allow redefining built-in rules"
+    ,opts $ Option ""  ["no-allow-redefine-rules"] (noArg $ \s -> s{shakeAllowRedefineRules = False}) "Forbid redefining built-in rules (default)"
     ,extr $ Option ""  ["no-build"] (noArg [NoBuild]) "Don't build anything."
     ,extr $ Option "C" ["directory"] (reqArg "DIRECTORY" $ \x -> [ChangeDirectory x]) "Change to DIRECTORY before doing anything."
 --    ,yes $ Option ""  ["cloud"] (reqArg "URL" $ \x s -> s{shakeCloud=shakeCloud s ++ [x]}) "HTTP server providing a cloud cache."
