@@ -99,15 +99,20 @@ actionBracketEx runOnSuccess alloc free act = do
     pure res
 
 -- | If an exception is raised by the 'Action', perform some 'IO' then reraise the exception.
+--   This function is implemented using 'actionBracket'.
 actionOnException :: Action a -> IO b -> Action a
 actionOnException act free = actionBracketEx False (pure ()) (const free) (const act)
 
 -- | After an 'Action', perform some 'IO', even if there is an exception.
+--   This function is implemented using 'actionBracket'.
 actionFinally :: Action a -> IO b -> Action a
 actionFinally act free = actionBracket (pure ()) (const free) (const act)
 
--- | Like `bracket`, but where the inner operation is of type 'Action'. Usually used as
+-- | Like 'bracket', but where the inner operation is of type 'Action'. Usually used as
 --   @'actionBracket' alloc free use@.
+--
+--   The @free@ action will be run masked. The cost of 'actionBracket' is _O(n log n)_
+--   in the number of simultaneous 'actionBracket' calls active in the program.
 actionBracket :: IO a -> (a -> IO b) -> (a -> Action c) -> Action c
 actionBracket = actionBracketEx True
 
@@ -257,7 +262,7 @@ traced msg act = do
     stop <- liftIO globalTimestamp
     let trace = newTrace msg start stop
     liftIO $ evaluate $ rnf trace
-    Action $ modifyRW $ \s -> s{localTraces = trace : localTraces s}
+    Action $ modifyRW $ \s -> s{localTraces = addTrace (localTraces s) trace}
     pure res
 
 
@@ -275,7 +280,7 @@ lintTrackRead ks = do
     Global{..} <- Action getRO
     when (isJust $ shakeLint globalOptions) $ do
         l@Local{..} <- Action getRW
-        deps <- liftIO $ concatMapM (listDepends globalDatabase) localDepends
+        deps <- liftIO $ concatMapM (listDepends globalDatabase) $ enumerateDepends localDepends
         let top = topStack localStack
 
         let condition1 k = top == Just k
@@ -317,7 +322,7 @@ lintTrackFinished = do
         let ignore k = any ($ k) localTrackAllows
 
         -- Read stuff
-        deps <- concatMapM (listDepends globalDatabase) localDepends
+        deps <- concatMapM (listDepends globalDatabase) $ enumerateDepends localDepends
         let used = Set.filter (not . ignore) $ Set.fromList localTrackRead
 
         -- check Read 4a
@@ -395,8 +400,14 @@ lookupDependencies db k = do
     pure $ depends r
 
 
--- | This rule should not be cached or recorded in the history because it makes use of untracked dependencies
---   (e.g. files in a system directory or items on the @$PATH@), or is trivial to compute locally.
+-- | This rule should not be saved to shared/cloud storage via 'shakeShare'.
+--   There are usually two reasons to call this function:
+--
+--   1. It makes use of untracked dependencies that are specific to this machine,
+--      e.g. files in a system directory or items on the @$PATH@.
+--   2. The rule is trivial to compute locally, so there is no point sharing it.
+--
+--   If you want the rule to not be cached at all, use 'alwaysRerun'.
 historyDisable :: Action ()
 historyDisable = Action $ modifyRW $ \s -> s{localHistory = False}
 
@@ -436,25 +447,26 @@ orderOnlyAction act = Action $ do
 --   Most people should use 'Development.Shake.newCache' instead.
 newCacheIO :: (Eq k, Hashable k) => (k -> Action v) -> IO (k -> Action v)
 newCacheIO (act :: k -> Action v) = do
-    var :: Var (Map.HashMap k (Fence IO (Either SomeException ([Depends],v)))) <- newVar Map.empty
+    var :: Var (Map.HashMap k (Fence IO (Either SomeException (DependsList,v)))) <- newVar Map.empty
     pure $ \key ->
         join $ liftIO $ modifyVar var $ \mp -> case Map.lookup key mp of
             Just bar -> pure $ (,) mp $ do
                 (offset, (deps, v)) <- actionFenceRequeue bar
-                Action $ modifyRW $ \s -> addDiscount offset $ s{localDepends = deps ++ localDepends s}
+                Action $ modifyRW $ \s -> addDiscount offset $ s{localDepends = addDepends (localDepends s) deps}
                 pure v
             Nothing -> do
                 bar <- newFence
                 pure $ (Map.insert key bar mp,) $ do
                     Local{localDepends=pre} <- Action getRW
+                    Action $ modifyRW $ \s -> s{localDepends = newDepends []}
                     res <- Action $ tryRAW $ fromAction $ act key
                     case res of
                         Left err -> do
                             liftIO $ signalFence bar $ Left err
                             Action $ throwRAW err
                         Right v -> do
-                            Local{localDepends=post} <- Action getRW
-                            let deps = dropEnd (length pre) post
+                            Local{localDepends=deps} <- Action getRW
+                            Action $ modifyRW $ \s -> s{localDepends = addDepends pre deps}
                             liftIO $ signalFence bar $ Right (deps, v)
                             pure v
 
@@ -478,6 +490,9 @@ unsafeExtraThread act = do
 
 
 -- | Execute a list of actions in parallel. In most cases 'need' will be more appropriate to benefit from parallelism.
+--   If the two types of 'Action' are different dependencies which ultimately boil down to 'apply',
+--   using 'Applicative' operations will still ensure the dependencies occur in parallel.
+--   The main use of this function is to run work that happens in an 'Action' in parallel.
 parallel :: [Action a] -> Action [a]
 -- Note: There is no parallel_ unlike sequence_ because there is no stack benefit to doing so
 parallel [] = pure []
