@@ -1,42 +1,55 @@
-{-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE ScopedTypeVariables, ConstraintKinds, GeneralizedNewtypeDeriving, ViewPatterns #-}
 
 module General.Extra(
     getProcessorCount,
     findGcc,
-    withResultType,
     whenLeft,
     randomElem,
     wrapQuote, showBracket,
-    withs,
+    withs, forNothingM,
     maximum', maximumBy',
+    unconcat,
     fastAt,
-    forkFinallyUnmasked,
+    zipExact, zipWithExact,
     isAsyncException,
-    doesFileExist_,
+    showDurationSecs,
+    usingLineBuffering,
+    doesFileExist_, doesDirectoryExist_,
+    usingNumCapabilities,
     removeFile_, createDirectoryRecursive,
-    catchIO, tryIO, handleIO
+    catchIO, tryIO, handleIO, handleSynchronous,
+    Located, Partial, callStackTop, callStackFull, withFrozenCallStack, callStackFromException,
+    Ver(..), makeVer,
+    QTypeRep(..),
+    NoShow(..)
     ) where
 
-import Control.Exception
+import Control.Exception.Extra
 import Data.Char
-import Data.List
-import System.Environment.Extra
+import Data.List.Extra
+import System.Environment
+import Development.Shake.FilePath
+import Control.DeepSeq
+import General.Cleanup
+import Data.Typeable
+import System.IO.Error
 import System.IO.Extra
+import System.Time.Extra
 import System.IO.Unsafe
 import System.Info.Extra
-import System.FilePath
 import System.Random
 import System.Directory
 import System.Exit
-import Control.Concurrent
+import Numeric.Extra
+import Foreign.Storable
+import Control.Concurrent.Extra
 import Data.Maybe
-import Data.Functor
+import Data.Hashable
 import Data.Primitive.Array
-import Control.Applicative
 import Control.Monad
 import Control.Monad.ST
 import GHC.Conc(getNumProcessors)
-import Prelude
+import GHC.Stack
 
 
 ---------------------------------------------------------------------
@@ -49,6 +62,14 @@ maximumBy' cmp = foldl1' $ \x y -> if cmp x y == GT then x else y
 maximum' :: Ord a => [a] -> a
 maximum' = maximumBy' compare
 
+newtype NoShow a = NoShow a
+instance Show (NoShow a) where show _ = "NoShow"
+
+unconcat :: [[a]] -> [b] -> [[b]]
+unconcat [] _ = []
+unconcat (a:as) bs = b1 : unconcat as b2
+    where (b1,b2) = splitAt (length a) bs
+
 
 ---------------------------------------------------------------------
 -- Data.List
@@ -77,10 +98,20 @@ fastAt xs = \i -> if i < 0 || i >= n then Nothing else Just $ indexArray arr i
         arr = runST $ do
             let n = length xs
             arr <- newArray n undefined
-            forM_ (zip [0..] xs) $ \(i,x) ->
+            forM_ (zipFrom 0 xs) $ \(i,x) ->
                 writeArray arr i x
             unsafeFreezeArray arr
 
+zipWithExact :: Partial => (a -> b -> c) -> [a] -> [b] -> [c]
+zipWithExact f = g
+    where
+        g [] [] = []
+        g (a:as) (b:bs) = f a b : g as bs
+        g _ _ = error "zipWithExacts: unequal lengths"
+
+zipExact :: Partial => [a] -> [b] -> [(a,b)]
+zipExact = zipWithExact (,)
+
 
 ---------------------------------------------------------------------
 -- System.Info
@@ -88,7 +119,7 @@ fastAt xs = \i -> if i < 0 || i >= n then Nothing else Just $ indexArray arr i
 {-# NOINLINE getProcessorCount #-}
 getProcessorCount :: IO Int
 -- unsafePefromIO so we cache the result and only compute it once
-getProcessorCount = let res = unsafePerformIO act in return res
+getProcessorCount = let res = unsafePerformIO act in pure res
     where
         act =
             if rtsSupportsBoundThreads then
@@ -96,10 +127,10 @@ getProcessorCount = let res = unsafePerformIO act in return res
             else do
                 env <- lookupEnv "NUMBER_OF_PROCESSORS"
                 case env of
-                    Just s | [(i,"")] <- reads s -> return i
+                    Just s | [(i,"")] <- reads s -> pure i
                     _ -> do
-                        src <- readFile' "/proc/cpuinfo" `catchIO` \_ -> return ""
-                        return $! max 1 $ length [() | x <- lines src, "processor" `isPrefixOf` x]
+                        src <- readFile' "/proc/cpuinfo" `catchIO` \_ -> pure ""
+                        pure $! max 1 $ length [() | x <- lines src, "processor" `isPrefixOf` x]
 
 
 -- Can you find a GCC executable? return a Bool, and optionally something to add to $PATH to run it
@@ -113,9 +144,9 @@ findGcc = do
                 Just ghc -> do
                     let gcc = takeDirectory (takeDirectory ghc) </> "mingw/bin/gcc.exe"
                     b <- doesFileExist_ gcc
-                    return $ if b then (True, Just $ takeDirectory gcc) else (False, Nothing)
-                _ -> return (False, Nothing)
-        _ -> return (isJust v, Nothing)
+                    pure $ if b then (True, Just $ takeDirectory gcc) else (False, Nothing)
+                _ -> pure (False, Nothing)
+        _ -> pure (isJust v, Nothing)
 
 
 
@@ -126,7 +157,26 @@ randomElem :: [a] -> IO a
 randomElem xs = do
     when (null xs) $ fail "General.Extra.randomElem called with empty list, can't pick a random element"
     i <- randomRIO (0, length xs - 1)
-    return $ xs !! i
+    pure $ xs !! i
+
+
+---------------------------------------------------------------------
+-- System.IO
+
+usingLineBuffering :: Cleanup -> IO ()
+usingLineBuffering cleanup = do
+    out <- hGetBuffering stdout
+    err <- hGetBuffering stderr
+    when (out /= LineBuffering || err /= LineBuffering) $ do
+        register cleanup $ hSetBuffering stdout out >> hSetBuffering stderr err
+        hSetBuffering stdout LineBuffering >> hSetBuffering stderr LineBuffering
+
+
+---------------------------------------------------------------------
+-- System.Time
+
+showDurationSecs :: Seconds -> String
+showDurationSecs = replace ".00s" "s" . showDuration . intToDouble . round
 
 
 ---------------------------------------------------------------------
@@ -136,15 +186,24 @@ withs :: [(a -> r) -> r] -> ([a] -> r) -> r
 withs [] act = act []
 withs (f:fs) act = f $ \a -> withs fs $ \as -> act $ a:as
 
+forNothingM :: Monad m => [a] -> (a -> m (Maybe b)) -> m (Maybe [b])
+forNothingM [] f = pure $ Just []
+forNothingM (x:xs) f = do
+    v <- f x
+    case v of
+        Nothing -> pure Nothing
+        Just v -> liftM (v:) `liftM` forNothingM xs f
+
 
 ---------------------------------------------------------------------
 -- Control.Concurrent
 
--- | Like 'forkFinally', but the inner thread is unmasked even if you started masked.
-forkFinallyUnmasked :: IO a -> (Either SomeException a -> IO ()) -> IO ThreadId
-forkFinallyUnmasked act cleanup =
-    mask_ $ forkIOWithUnmask $ \unmask ->
-        try (unmask act) >>= cleanup
+usingNumCapabilities :: Cleanup -> Int -> IO ()
+usingNumCapabilities cleanup new = when rtsSupportsBoundThreads $ do
+    old <- getNumCapabilities
+    when (old /= new) $ do
+        register cleanup $ setNumCapabilities old
+        setNumCapabilities new
 
 
 ---------------------------------------------------------------------
@@ -158,24 +217,35 @@ isAsyncException e
     | otherwise = False
 
 catchIO :: IO a -> (IOException -> IO a) -> IO a
-catchIO = Control.Exception.catch -- GHC 7.4 has catch in the Prelude as well
+catchIO = catch
 
 tryIO :: IO a -> IO (Either IOException a)
 tryIO = try
 
 handleIO :: (IOException -> IO a) -> IO a -> IO a
-handleIO = flip catchIO
+handleIO = handle
 
+handleSynchronous :: (SomeException -> IO a) -> IO a -> IO a
+handleSynchronous = handleBool (not . isAsyncException)
 
 ---------------------------------------------------------------------
 -- System.Directory
 
 doesFileExist_ :: FilePath -> IO Bool
-doesFileExist_ x = doesFileExist x `catchIO` \_ -> return False
+doesFileExist_ x = doesFileExist x `catchIO` \_ -> pure False
+
+doesDirectoryExist_ :: FilePath -> IO Bool
+doesDirectoryExist_ x = doesDirectoryExist x `catchIO` \_ -> pure False
 
 -- | Remove a file, but don't worry if it fails
 removeFile_ :: FilePath -> IO ()
-removeFile_ x = removeFile x `catchIO` \_ -> return ()
+removeFile_ x =
+    removeFile x `catchIO` \e ->
+        when (isPermissionError e) $ handleIO (\_ -> pure ()) $ do
+            perms <- getPermissions x
+            setPermissions x perms{readable = True, searchable = True, writable = True}
+            removeFile x
+
 
 -- | Like @createDirectoryIfMissing True@ but faster, as it avoids
 --   any work in the common case the directory already exists.
@@ -193,8 +263,51 @@ whenLeft x f = either f (const $ pure ()) x
 
 
 ---------------------------------------------------------------------
--- Data.Proxy
+-- Data.CallStack
 
--- Should be Proxy, but that's not available in older GHC 7.6 and before
-withResultType :: (Maybe a -> a) -> a
-withResultType f = f Nothing
+type Located = Partial
+
+callStackTop :: Partial => String
+callStackTop = withFrozenCallStack $ headDef "unknown location" callStackFull
+
+callStackFull :: Partial => [String]
+callStackFromException :: SomeException -> ([String], SomeException)
+
+
+-- | Invert 'prettyCallStack', which GHC pre-applies in certain cases
+parseCallStack = reverse . map trimStart . drop1 . lines
+
+callStackFull = parseCallStack $ prettyCallStack $ popCallStack callStack
+
+callStackFromException (fromException -> Just (ErrorCallWithLocation msg loc)) = (parseCallStack loc, toException $ ErrorCall msg)
+callStackFromException e = ([], e)
+
+
+---------------------------------------------------------------------
+-- Data.Version
+
+-- | A version number that indicates change, not ordering or compatibility.
+--   Always presented as an 'Int' to the user, but a newtype inside the library for safety.
+newtype Ver = Ver Int
+    deriving (Show,Eq,Storable)
+
+makeVer :: String -> Ver
+makeVer = Ver . hash
+
+
+---------------------------------------------------------------------
+-- Data.Typeable
+
+-- | Like TypeRep, but the Show includes enough information to be unique
+--   so I can rely on @a == b === show a == show b@.
+newtype QTypeRep = QTypeRep {fromQTypeRep :: TypeRep}
+    deriving (Eq,Hashable,NFData)
+
+instance Show QTypeRep where
+    -- Need to show enough so that different types with the same names don't clash
+    -- But can't show too much or the history is not portable https://github.com/ndmitchell/shake/issues/670
+    show (QTypeRep x) = f x
+        where
+            f x = ['(' | xs /= []] ++ (unwords $ g c : map f xs) ++ [')' | xs /= []]
+                where (c, xs) = splitTyConApp x
+            g x = tyConModule x ++ "." ++ tyConName x
