@@ -4,7 +4,8 @@ module Test.Benchmark(main) where
 import General.GetOpt
 import Development.Shake
 import Test.Type
-import Text.Read.Extra
+import Text.Read
+import Data.List.Extra
 import Development.Shake.FilePath
 
 
@@ -13,9 +14,9 @@ opts = [Option "" ["depth"  ] (ReqArg (fmap Depth   . readEither) "INT") ""
        ,Option "" ["breadth"] (ReqArg (fmap Breadth . readEither) "INT") ""]
 
 -- | Given a breadth and depth come up with a set of build files
-main = shakeTest test opts $ \opts -> do
-    let depth   = last $ error "Missing --depth"   : [x | Depth   x <- opts]
-    let breadth = last $ error "Missing --breadth" : [x | Breadth x <- opts]
+main = testBuildArgs test opts $ \opts -> do
+    let depth   = lastDef 75 [x | Depth   x <- opts]
+    let breadth = lastDef 75 [x | Breadth x <- opts]
 
     want ["0." ++ show i | i <- [1..breadth]]
     "*" %> \out -> do
@@ -26,5 +27,5 @@ main = shakeTest test opts $ \opts -> do
 test build = do
     -- these help to test the stack limit
     build ["clean"]
-    build ["--breadth=75","--depth=75"]
-    build ["--breadth=75","--depth=75"]
+    build []
+    build []
