@@ -10,8 +10,7 @@ import Test.Type
 import Control.Monad.Extra
 import Data.Char
 import Data.List.Extra
-import System.Info
-import Data.Version.Extra
+import System.Info.Extra
 
 
 newtype GhcPkg = GhcPkg () deriving (Show,Typeable,Eq,Hashable,Binary,NFData)
@@ -20,7 +19,8 @@ newtype GhcFlags = GhcFlags () deriving (Show,Typeable,Eq,Hashable,Binary,NFData
 type instance RuleResult GhcPkg = [String]
 type instance RuleResult GhcFlags = [String]
 
-main = testBuild defaultTest $ do
+-- Doesn't work on the Windows CI for reasons I don't understand
+main = testBuild (notWindowsCI . defaultTest) $ do
     let moduleToFile ext xs = replace "." "/" xs <.> ext
     want ["Main" <.> exe]
 
@@ -35,6 +35,7 @@ main = testBuild defaultTest $ do
         map ("-package=" ++) <$> readFileLines ".pkgs"
 
     let ghc args = do
+            trackAllow ["**/package.cache", "**/.ghc.environment.*"]
             -- since ghc-pkg includes the ghc package, it changes if the version does
             ghcPkg $ GhcPkg ()
             flags <- ghcFlags $ GhcFlags ()
@@ -82,10 +83,8 @@ hsImports xs = [ takeWhile (\x -> isAlphaNum x || x `elem` "._") $ dropWhile (no
 
 -- FIXME: Should actually parse the list from the contents of the .cabal file
 cabalBuildDepends :: String -> [String]
-cabalBuildDepends _ = packages ++ ["unix" | os /= "mingw32"]
+cabalBuildDepends _ = packages ++ ["unix" | not isWindows]
 
-packages = words
-    ("base transformers binary unordered-containers hashable heaps time bytestring primitive " ++
-     "filepath directory process deepseq random utf8-string extra js-dgtable js-jquery js-flot filepattern") ++
-    ["old-time" | compilerVersion < makeVersion [7,6]] ++
-    ["semigroups" | compilerVersion < makeVersion [8,0]]
+packages = words $
+    "base transformers binary unordered-containers hashable heaps time bytestring primitive " ++
+    "filepath directory process deepseq random utf8-string extra js-dgtable js-jquery js-flot filepattern"
